@@ -5,6 +5,7 @@ __all__ = [
     "InMemoryQueryExecutorBase",
     "FeatureComputeBase",
     "MetricsBase",
+    "AnomaliesBase"
 ]
 
 from .fileops import *
@@ -13,3 +14,4 @@ from .inmemory_query_executor import *
 from .notebook_executor import *
 from .feature_compute import *
 from .metrics import *
+from .anomalies import *
