@@ -1080,9 +1080,9 @@ def bootstrap(component, path, templatepath):
         prepare_dashboard(path, templatepath)
     elif component in ["tasklib", "task-lib"]:
         prepare_tasklib(path, templatepath)
-    elif component in ["taskconf", "task-conf"]:
+    elif component in ["taskconf", "task-conf", "task"]:
         prepare_taskconf(path, templatepath)
-    elif component in ["pipelineconf", "pipeline-conf"]:
+    elif component in ["pipelineconf", "pipeline-conf", "pipeline"]:
         prepare_pipelineconf(path, templatepath)
     elif component in ["workspace", "usecase"]:
         prepare_workspace(path)
