@@ -18,6 +18,11 @@ import pyfiglet
 # thirdparty
 import click
 from pythonjsonlogger import jsonlogger
+from prompt_toolkit import prompt
+
+# Disable warnings
+import urllib3
+urllib3.disable_warnings()
 
 # self
 import enrichsdk
@@ -29,12 +34,59 @@ from enrichsdk import (
     featurestore as fslib,
     utils,
 )
+from enrichsdk.package.validators import GenericDescValidator
+
+class AliasedGroup(click.Group):
+
+    def get_command(self, ctx, cmd_name):
+
+        if cmd_name == 'bootstrap':
+            cmd_name = 'init'
+
+        rv = click.Group.get_command(self, ctx, cmd_name)
+        if rv is not None:
+            return rv
+        print(self.list_commands(ctx))
+        print(cmd_name)
+        matches = [x for x in self.list_commands(ctx)  if x.startswith(cmd_name)]
+        if not matches:
+            return None
+        elif len(matches) == 1:
+            return click.Group.get_command(self, ctx, matches[0])
+        ctx.fail('Too many matches: %s' % ', '.join(sorted(matches)))
 
 
-@click.group()
+def get_backend_obj(ctx, backend):
+    try:
+        filename = ctx.obj["config"]
+        config = json.load(open(filename))
+    except:
+        traceback.print_exc()
+        raise Exception("Invalid config file: {}".format(ctx.obj["config"]))
+
+    backends = config["backends"]
+
+    relevant = None
+    for b in backends:
+        if (b.get("server", None) == backend) or (b.get("name", None) == backend):
+            relevant = b
+            break
+
+    if relevant is None:
+        raise Exception("Invalid backend name: {}".format(name))
+
+    # Decode the key before accessing
+    for col in ['key', 'htpasswd_user', 'htpasswd_pwd']:
+        value = base64.b64decode(relevant[col].encode("utf-8")).decode("utf-8")
+        relevant[col] = value
+
+    backend = sdkapi.Backend(relevant)
+    return backend
+
+@click.group(cls=AliasedGroup)
 def process():
     """
-    Bootstrap/test/install Enrich modules and access server
+    init/test/install Enrich modules and access server
 
     \b
     Getting started:
@@ -44,7 +96,7 @@ def process():
 
     \b
     Development:
-       bootstrap:  Bootstrap modules including transforms
+       init:  Bootstrap modules including transforms*
        test:      Test transforms, manage datasets
 
     \b
@@ -58,6 +110,8 @@ def process():
     \b
     Helpers:
        show-log:  Pretty print log output
+
+    *Command used to be called bootstrap
     """
     pass
 
@@ -161,14 +215,14 @@ def _start():
         {
             "cmd": [
                 "# if acme is already populated",
-                "enrichpkg bootstrap -c repo -p ."
+                "enrichpkg init repo -p ."
             ],
-            "description": "[OPTIONAL] Bootstrap the repo, if not already done. In case of Acme you dont have tEnrich requires a particular structure to the repository. You will notice __init__.py and enrich.json. These are files that tell the Enrich system what is present where.",
+            "description": "[OPTIONAL] Initialize the repo, if not already done. In case of Acme you dont have tEnrich requires a particular structure to the repository. You will notice __init__.py and enrich.json. These are files that tell the Enrich system what is present where.",
         },
         {
             "cmd": [
                 "# Typically usecases have first letter in capital",
-                "enrichpkg bootstrap -c usecase -p Marketing",
+                "enrichpkg init usecase -p Marketing",
             ],
             "description": "[OPTIONAL] The code is organized in terms of projects or usecase groups. Create one usecase, say Marketing. In case of acme, you dont have to do this. Just look around."
         },
@@ -179,23 +233,23 @@ def _start():
             "description": "Change to usecase"
         },
         {
-            "description": "Bootstrap a simple transform. Transform is a python module. It could be a single file, directory or a full package.",
+            "description": "Initialize a simple transform. Transform is a python module. It could be a single file, directory or a full package.",
             "cmd": [
                 "# Will create an example hello world script",
-                "enrichpkg bootstrap transform-helloworld -p transforms/helloworld.py",
+                "enrichpkg init transform-helloworld -p transforms/helloworld.py",
             ]
         },
         {
-            "description": "[OPTIONAL] You could do more complex transforms. You can bootstrap a python module directory or a full package. You can come back to this later on",
+            "description": "[OPTIONAL] You could do more complex transforms. You can initialize a python module directory or a full package. You can come back to this later on",
             "cmd": [
                 "# Will create a more simple python script that is more complex than helloworld",
-                "enrichpkg bootstrap transform-simple -p transforms/simplefile.py",
+                "enrichpkg init transform-simple -p transforms/simplefile.py",
                 "",
                 "# Will create a python module (a full directory)",
-                "enrichpkg bootstrap transform-simple -p transforms/simpledir",
+                "enrichpkg init transform-simple -p transforms/simpledir",
                 "",
                 "# Will create a python package. Dont use this unless you are sure",
-                "enrichpkg bootstrap transform-package -p transforms/simplepackage",
+                "enrichpkg init transform-package -p transforms/simplepackage",
             ],
         },
         {
@@ -220,9 +274,9 @@ def _start():
         },
         {
             "cmd": [
-                "enrichpkg bootstrap pipeline-conf -p pipelines/conf/helloworld.py"
+                "enrichpkg init pipeline -p pipelines/conf/helloworld.py"
             ],
-            "description": "Transforms are like plugins. They have to be incorporated into a pipeline to ensure that it runs on server. Bootstrap a pipeline. Note the path to the pipelines. They have a particular structure as described in the SDK and at particular location. The Enrich execution engine searches these paths",
+            "description": "Transforms are like plugins. They have to be incorporated into a pipeline to ensure that it runs on server. Initialize a pipeline. Note the path to the pipelines. They have a particular structure as described in the SDK and at particular location. The Enrich execution engine searches these paths",
         },
         {
             "cmd": ["enrichpkg test conf --capture pipelines/conf/helloworld.py"],
@@ -231,7 +285,7 @@ def _start():
         {
             "cmd": [
                 "# prefect is the default workflow engine",
-                "enrichpkg bootstrap prefectjob -p workflows/prefect/daily.py",
+                "enrichpkg init prefectjob -p workflows/prefect/daily.py",
             ],
             "description": "A workflow allows us to combine multiple pipelines and multiple runs of each into a single activity. We use prefect for the same. We could use something else. We dont need prefect cloud though. We will run the workflow from our scheduler"
         },
@@ -261,7 +315,7 @@ def _start():
         print("")
 
 
-@click.command("bootstrap", hidden=True)
+@click.command("init", hidden=True)
 @click.argument("component")
 @click.option(
     "--path", "-p", required=False, default=None, help="Directory where the files should be stored"
@@ -274,12 +328,14 @@ def _start():
     help="Template to override the defaults",
 )
 @click.option("--context", required=False, default=None, help="Environment file")
-def bootstrap_(component, path, template, context):
+def init_(component, path, template, context):
     """
-    Bootstrap a fresh module
+    Bootstrap/initialize a fresh module
 
     Modules currently supported include transform, pipeline and asset. Use
-    help to list possible components  (enrichpkg bootstrap help)
+    help to list possible components::
+
+         enrichpkg init help*
 
     For now only the transforms and models are testable locally
     without the need for a test server.
@@ -289,6 +345,8 @@ def bootstrap_(component, path, template, context):
 
     A custom template can be provided as well.
 
+    * init command used to be called bootstrap
+
     """
 
     components = [
@@ -298,10 +356,10 @@ def bootstrap_(component, path, template, context):
         ["transform-simple", "A transform structured as a python module"],
         ["transform-helloworld", "A simple hello world transform for study"],
         ["transform-query", "A  transform that uses a pre-existing database querying module"],
-        ["transform-iris", "A  transform that uses a pre-existing database querying module"],
-        ["asset", "A reusable library"],
+        ["transform-iris", "Simple Iris post-processing example"],
+        ["asset", "A reusable library "],
         ["dashboard", "A django app that fits the Enrich Dashboard framework"],
-        ["pipeline-conf", "A pipeline configuration"],
+        ["pipeline", "A pipeline configuration"],
         ["prefectjob", "Prefect workflow template"],
         ["rscript", "R-Script template"],
         ["pyscript", "A python script template"]
@@ -312,7 +370,7 @@ def bootstrap_(component, path, template, context):
         for c in components:
             print(f"   {c[0]:20}: {c[1]}")
         print("\nUse the component name like this")
-        print("    enrichpkg bootstrap repo -p <path>")
+        print("    enrichpkg init repo -p <path>")
         print("")
         print("You can define your own templates as well. Please use the -t option\n")
         return
@@ -493,7 +551,7 @@ def test_pkg(ctx, pkgdir, capture):
     context = ctx.obj["context"]
 
     if not package.checkenv(validate_only=True):
-        print("Try: enrichpkg checkenv")
+        print("Try: enrichpkg env check")
         raise Exception("Environment incomplete")
 
     package.test_transform(pkgdir, capture, context)
@@ -511,7 +569,7 @@ def test_task(ctx, taskdir, capture):
     context = ctx.obj["context"]
 
     if not package.checkenv(validate_only=True):
-        print("Try: enrichpkg checkenv")
+        print("Try: enrichpkg env check")
         raise Exception("Environment incomplete")
 
     package.test_task(taskdir, capture, context)
@@ -529,7 +587,7 @@ def test_conf(ctx, spec, capture):
     context = ctx.obj["context"]
 
     if not package.checkenv(validate_only=True):
-        print("Try: enrichpkg checkenv")
+        print("Try: enrichpkg env check")
         raise Exception("Environment incomplete")
 
     package.test_conf(spec, capture, context)
@@ -547,7 +605,7 @@ def testdata(ctx, spec, capture):
     spec could be transform or a spec file
     """
     if not package.checkenv(validate_only=True):
-        print("Try: enrichpkg checkenv")
+        print("Try: enrichpkg env check")
         raise Exception("Environment incomplete")
 
     ctx.ensure_object(dict)
@@ -627,7 +685,7 @@ def api(ctx, config):
     """
 
     if not package.checkenv(validate_only=True):
-        print("Try: enrichpkg checkenv")
+        print("Try: enrichpkg env check")
         raise Exception("Environment incomplete")
 
     alt = os.path.expandvars("$ENRICH_ETC/api.json")
@@ -709,6 +767,11 @@ def _config_init(ctx):
         "services": [],
     }
     filename = ctx.obj["config"]
+    if os.path.exists(filename):
+        print("API access configuration file already exists. Please remove it first")
+        print("See:", filename)
+        return
+
     with open(filename, "w") as fd:
         fd.write(json.dumps(default, indent=4))
 
@@ -752,9 +815,8 @@ def _config_list(ctx):
 @config.command("backend-add")
 @click.argument("name")
 @click.argument("server")
-@click.argument("key")
 @click.pass_context
-def _backend_add(ctx, name, server, key):
+def _backend_add(ctx, name, server):
     """
     Add backend
     """
@@ -765,8 +827,18 @@ def _backend_add(ctx, name, server, key):
         traceback.print_exc()
         raise Exception("Invalid config file: {}".format(ctx.obj["config"]))
 
+    key = prompt(">API Key: ")
+    htpasswd_user = prompt(">HTPasswd Username: ")
+    htpasswd_pwd = prompt(">HTPasswd Password: ",)
+
+    if len(key) == 0 or len(htpasswd_user) == 0 or len(htpasswd_pwd) == 0:
+        print("Empty APIKey or HTPasswd username or password")
+        return
+
     # Encode the key before storing
     key = base64.b64encode(key.encode("utf-8")).decode("utf-8")
+    htpasswd_user  = base64.b64encode(htpasswd_user.encode("utf-8")).decode("utf-8")
+    htpasswd_pwd  = base64.b64encode(htpasswd_pwd.encode("utf-8")).decode("utf-8")
 
     backends = config["backends"]
     missing = True
@@ -775,11 +847,19 @@ def _backend_add(ctx, name, server, key):
             b["name"] = name
             b["server"] = server
             b["key"] = key
+            b["htpasswd_user"] = htpasswd_user
+            b["htpasswd_pwd"] = htpasswd_pwd
             missing = False
             break
 
     if missing:
-        backends.append({"name": name, "server": server, "key": key})
+        backends.append({
+            "name": name,
+            "server": server,
+            "key": key,
+            "htpasswd_user": htpasswd_user,
+            "htpasswd_pwd": htpasswd_pwd,
+        })
 
     with open(filename, "w") as fd:
         fd.write(json.dumps(config, indent=4))
@@ -892,6 +972,99 @@ def show(ctx, backend):
     """
     Access a backend
     """
+    ctx.obj["backend"] = get_backend_obj(ctx, backend)
+
+
+@click.command("health")
+@click.argument("backend")
+@click.pass_context
+def _health(ctx, backend):
+    """
+    Show system health
+    """
+
+    backend = get_backend_obj(ctx, backend)
+    result = backend.health()
+
+    status = result["status"]
+    if status in ["failure", "error"]:
+        print("Failure while accessing backend")
+        if "error" in result:
+            print("Reason:", result["error"])
+        return
+
+    sdkapi.draw_dict(backend, "Health", result)
+
+
+@click.command("app-list")
+@click.argument("backend")
+@click.pass_context
+def _app_list(ctx, backend):
+    """
+    Show apps available
+    """
+
+    backend = get_backend_obj(ctx, backend)
+    try:
+        result = backend.app_list()
+        print(json.dumps(result, indent=4))
+    except:
+        traceback.print_exc()
+
+@click.command("app-detail")
+@click.argument("backend")
+@click.argument("name")
+@click.pass_context
+def _app_detail(ctx, backend,name):
+    """
+    Show detail of an app
+    """
+
+    backend = get_backend_obj(ctx, backend)
+    try:
+        result = backend.app_detail(name)
+        print(json.dumps(result, indent=4))
+    except:
+        traceback.print_exc()
+
+
+@click.command("pipeline-runs")
+@click.argument("backend")
+@click.argument("usecase")
+@click.argument("pipeline")
+@click.pass_context
+def _run_list(ctx, backend, usecase, pipeline):
+    """
+    Show available runs for pipeline
+    """
+
+    backend = get_backend_obj(ctx, backend)
+    result = backend.run_list(usecase, pipeline)
+
+    status = result["status"]
+    if status in ["failure", "error"]:
+        print("Failure while accessing backend")
+        if "error" in result:
+            print("Reason:", result["error"])
+        if "message" in result:
+            print("Reason:", result["message"])
+        return
+
+    result["data"] = sorted(result["data"], key=lambda r: r["start_time"], reverse=True)
+
+    print(json.dumps(result['data'], indent=4))
+    #columns = ["runid", "status", "start_time", "end_time"]
+    #sdkapi.draw(
+    #    backend, "run-list", result['data'], columns, {"Usecase": usecase, "Pipeline": pipeline}
+    #)
+
+@click.group("scheduler")
+@click.argument("backend")
+@click.pass_context
+def scheduler(ctx, backend):
+    """
+    Access scheduler at backend
+    """
     try:
         filename = ctx.obj["config"]
         config = json.load(open(filename))
@@ -911,144 +1084,36 @@ def show(ctx, backend):
         raise Exception("Invalid backend name: {}".format(name))
 
     # Decode the key before accessing
-    key = base64.b64decode(relevant["key"].encode("utf-8")).decode("utf-8")
-    relevant["key"] = key
+    for col in ['key', 'htpasswd_user', 'htpasswd_pwd']:
+        value = base64.b64decode(relevant[col].encode("utf-8")).decode("utf-8")
+        relevant[col] = value
 
     backend = sdkapi.Backend(relevant)
     ctx.obj["backend"] = backend
 
 
-@show.command("status")
-@click.option("--offset", default=0)
-@click.option("--page", default=20)
+@scheduler.command("list")
 @click.pass_context
-def _status(ctx, offset, page):
+def _scheduler_list(ctx):
     """
-    Execution status
+    Show the scheduled jobs
     """
+    backend = ctx.obj['backend']
+    try:
+        result = backend.scheduler_jobs_list()
 
-    backend = ctx.obj["backend"]
-    result = backend.status(offset, page)
+        jobs = result['jobs']
 
-    status = result["status"]
-    if status in ["failure", "error"]:
-        print("Failure while accessing backend")
-        if "error" in result:
-            print("Reason:", result["error"])
-        return
-
-    columns = ["usecase", "nature", "name", "status", "runid", "start_time", "duration"]
-    sdkapi.draw(backend, "status", result, columns)
-
-
-@show.command("usecases")
-@click.pass_context
-def _usecases(ctx):
-    """
-    Show available usecases
-    """
-
-    backend = ctx.obj["backend"]
-    result = backend.usecases()
-
-    status = result["status"]
-    if status in ["failure", "error"]:
-        print("Failure while accessing backend")
-        if "error" in result:
-            print("Reason:", result["error"])
-        return
-
-    columns = ["customer", "name", "description"]
-    sdkapi.draw(backend, "usecases", result, columns)
-
-
-@show.command("health")
-@click.pass_context
-def _health(ctx):
-    """
-    Show system health
-    """
-
-    backend = ctx.obj["backend"]
-    result = backend.health()
-
-    status = result["status"]
-    if status in ["failure", "error"]:
-        print("Failure while accessing backend")
-        if "error" in result:
-            print("Reason:", result["error"])
-        return
-
-    sdkapi.draw_health(backend, "health", result)
-
-
-@show.command("pipelines")
-@click.argument("usecase")
-@click.pass_context
-def _pipelines(ctx, usecase):
-    """
-    Show available pipelines
-    """
-
-    backend = ctx.obj["backend"]
-    result = backend.pipelines(usecase)
-
-    status = result["status"]
-    if status in ["failure", "error"]:
-        print("Failure while accessing backend")
-        if "error" in result:
-            print("Reason:", result["error"])
-        return
-
-    data = result["data"]
-    for d in data:
-        d["description"] = d["description"][:20]
-        runs = sorted(d["runs"], reverse=True)
-        if len(runs) > 0:
-            d["lastrun"] = runs[0]
-            d["runs"] = len(runs)
-        else:
-            d["lastrun"] = ""
-            d["runs"] = 0
-
-    columns = ["usecase", "name", "description", "path", "runs", "lastrun"]
-    sdkapi.draw(
-        backend,
-        "pipelines",
-        result,
-        columns,
-        {
-            "Usecase": usecase,
-        },
-    )
-
-
-@show.command("run-list")
-@click.argument("usecase")
-@click.argument("pipeline")
-@click.pass_context
-def _runs(ctx, usecase, pipeline):
-    """
-    Show available runs for pipeline
-    """
-
-    backend = ctx.obj["backend"]
-    result = backend.run_list(usecase, pipeline)
-
-    status = result["status"]
-    if status in ["failure", "error"]:
-        print("Failure while accessing backend")
-        if "error" in result:
-            print("Reason:", result["error"])
-        return
-
-    result["data"] = sorted(result["data"], key=lambda r: r["start_time"], reverse=True)
-
-    columns = ["runid", "status", "start_time", "end_time"]
-    sdkapi.draw(
-        backend, "run-list", result, columns, {"Usecase": usecase, "Pipeline": pipeline}
-    )
+        #print(json.dumps(jobs, indent=4))
+        for j in jobs:
+            schedule = "%(minute)s %(hour)s %(day)s %(month)s %(day_of_week)s" % j
+            j['cron'] = schedule
+            for col in ['minute', 'hour', 'day', 'month', 'day_of_week']:
+                j.pop(col, None)
 
+        sdkapi.draw_dict(backend, "Scheduler List", jobs)
+    except:
+        traceback.print_exc()
 
 @show.command("run-detail")
 @click.argument("usecase")
@@ -1325,17 +1390,21 @@ def _spec_search(ctx, debug, params):
 
     print(json.dumps(response, indent=4))
 
-
+#api.add_command(show)
+#api.add_command(featurestore)
 api.add_command(config)
-api.add_command(show)
-api.add_command(featurestore)
+api.add_command(_run_list)
+api.add_command(_health)
+api.add_command(scheduler)
+api.add_command(_app_list)
+api.add_command(_app_detail)
 
 ###################################
 # Add to process
 ###################################
 process.add_command(_version)
 process.add_command(_start)
-process.add_command(bootstrap_)
+process.add_command(init_)
 process.add_command(envsetup)
 process.add_command(api)
 process.add_command(test)
